# -*- coding: utf-8 -*-

# Copyright (c) 2007 - 2008 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the CharTables plugin.
"""

import os

from PyQt4.QtCore import QObject, SIGNAL, QString, QTranslator

from KdeQt.KQApplication import e4App

from E4Gui.E4Action import E4Action

import UI.PixmapCache

# Start-of-Header
name = "CharTables Plugin"
author = "Detlev Offenbach <detlev@die-offenbachs.de>"
autoactivate = True
deactivateable = True
version = "4.1.4"
className = "CharTablesPlugin"
packageName = "CharTables"
shortDescription = "Dialog showing various character tables."
longDescription = """This plugin implements a dialog showing various character"""\
    """tables. The dialog was created by Jürgen Urner."""
needsRestart = False
# End-of-Header

error = QString("")

class CharTablesPlugin(QObject):
    """
    Class implementing the CharTables plugin.
    """
    def __init__(self, ui):
        """
        Constructor
        
        @param ui reference to the user interface object (UI.UserInterface)
        """
        QObject.__init__(self, ui)
        self.__ui = ui
        self.__initialize()
        
        self.__translator = None
        self.__loadTranslator()
    
    def __initialize(self):
        """
        Private slot to (re)initialize the plugin.
        """
        self.__chartablesDlg = None
        self.__chartablesAct = None
        self.__toolbarSeparator = None
    
    def activate(self):
        """
        Public method to activate this plugin.
        
        @return tuple of None and activation status (boolean)
        """
        global error
        
        icon = os.path.join(os.path.dirname(__file__), 
                            "CharTables", "chartables.png")
        self.__chartablesAct = E4Action(self.trUtf8('Character Tables'),
                UI.PixmapCache.getIcon(icon),
                self.trUtf8('Character &Tables...'), 0, 0, self, 'chartables')
        self.__chartablesAct.setStatusTip(self.trUtf8('Show character tables dialog'))
        self.__chartablesAct.setWhatsThis(self.trUtf8(
            """<b>Character Tables</b>"""
            """<p>Open a dialog to show and insert special characters.</p>"""
        ))
        self.connect(self.__chartablesAct, SIGNAL('triggered()'), self.__showChartables)
        
        try:
            toolbarManager = e4App().getObject("ToolbarManager")
            toolbarManager.addAction(self.__chartablesAct, 
                                     self.trUtf8("CharTables Plugin"))
        except KeyError:
            tb = self.__ui.getToolbar("tools")[1]
            if tb is not None:
                self.__toolbarSeparator = tb.addSeparator()
                tb.addAction(self.__chartablesAct)
        
        self.connect(self.__ui, SIGNAL("showMenu"), self.__showMenu)
        
        error = QString("")
        return None, True

    def deactivate(self):
        """
        Public method to deactivate this plugin.
        """
        try:
            toolbarManager = e4App().getObject("ToolbarManager")
            toolbarManager.removeAction(self.__chartablesAct)
        except KeyError:
            tb = self.__ui.getToolbar("tools")[1]
            if tb is not None:
                tb.removeAction(self.__chartablesAct)
                tb.removeAction(self.__toolbarSeparator)
        
        if self.__chartablesDlg is not None:
            viewmanager = e4App().getObject("ViewManager")
            self.disconnect(viewmanager, SIGNAL('editorOpened'),
                self.__chartablesDlg.editorOpened)
            self.disconnect(viewmanager, SIGNAL('lastEditorClosed'),
                self.__chartablesDlg.lastEditorClosed)
        
        self.__initialize()

    def __loadTranslator(self):
        """
        Private method to load the translation file.
        """
        if self.__ui is not None:
            loc = self.__ui.getLocale()
            if loc and loc != "C":
                locale_dir = \
                    os.path.join(os.path.dirname(__file__), "CharTables", "i18n")
                translation = "chartables_%s" % loc
                translator = QTranslator(None)
                loaded = translator.load(translation, locale_dir)
                if loaded:
                    self.__translator = translator
                    e4App().installTranslator(self.__translator)
                else:
                    print "Warning: translation file '%s' could not be loaded." % \
                        translation
                    print "Using default."
    
    def __showMenu(self, menuName, menu):
        """
        Private slot called, when the the tools menu is about to be shown.
        
        @param menuName name of the menu to be shown (string)
        @param menu reference to the menu (QMenu)
        """
        if menuName == "Tools":
            if self.__chartablesAct and \
               self.__chartablesAct not in menu.actions():
                menu.addAction(self.__chartablesAct)
    
    def __showChartables(self):
        """
        Private slot to show the character tables dialog.
        """
        if self.__chartablesDlg is None:
            # generate the character tables dialog
            from CharTables.chartables.e4Chartables import E4CharTablesDialog
            self.__chartablesDlg = E4CharTablesDialog(self.__ui.getLocale())
            viewmanager = e4App().getObject("ViewManager")
            self.connect(viewmanager, SIGNAL('editorOpened'),
                self.__chartablesDlg.editorOpened)
            self.connect(viewmanager, SIGNAL('lastEditorClosed'),
                self.__chartablesDlg.lastEditorClosed)
        self.__chartablesDlg.show()

