# -*- coding: utf-8 -*-

# Copyright (c) 2008 - 2009 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the Django project plugin.
"""

import sys
import os
import glob
import fnmatch

from PyQt4.QtCore import QObject, SIGNAL, QTranslator, QString, QStringList, QVariant
from PyQt4.QtGui import QApplication

from KdeQt.KQApplication import e4App

from Project.ProjectBrowser import SourcesBrowserFlag, FormsBrowserFlag, \
    ResourcesBrowserFlag, TranslationsBrowserFlag, InterfacesBrowserFlag, \
    OthersBrowserFlag, AllBrowsersFlag

import Preferences

from ProjectDjango.Project import Project

# Start-of-Header
name = "Django Project Plugin"
author = "Detlev Offenbach <detlev@die-offenbachs.de>"
autoactivate = True
deactivateable = True
version = "2.1.0"
className = "ProjectDjangoPlugin"
packageName = "ProjectDjango"
shortDescription = "Project support for Django projects."
longDescription = """This plugin implements project support for Django projects."""
needsRestart = False
# End-of-Header

error = QString("")

djangoPluginObject = None

def apiFiles(language):
    """
    Module function to return the API files made available by this plugin.
    
    @return list of API filenames (list of string)
    """
    if language == "Python":
        apisDir = \
            os.path.join(os.path.dirname(__file__), "ProjectDjango", "APIs")
        apis = glob.glob(os.path.join(apisDir, '*.api'))
    else:
        apis = []
    return apis

def createDjangoPage(configDlg):
    """
    Module function to create the Django configuration page.
    
    @return reference to the configuration page
    """
    global djangoPluginObject
    from ProjectDjango.ConfigurationPage.DjangoPage import DjangoPage
    page = DjangoPage(djangoPluginObject)
    return page
    
def getConfigData():
    """
    Module function returning data as required by the configuration dialog.
    
    @return dictionary containing the relevant data
    """
    if e4App().getObject("UserInterface").versionIsNewer('4.1.99', '20080731'):
        return {
            "djangoPage" : \
                [QApplication.translate("ProjectDjangoPlugin", "Django"), 
                 os.path.join("ProjectDjango", "icons", 
                              "django.png"),
                 createDjangoPage, None, None],
        }
    else:
        return {}

def prepareUninstall():
    """
    Module function to prepare for an uninstallation.
    """
    django = ProjectDjangoPlugin(None)
    django.prepareUninstall()
    
class ProjectDjangoPlugin(QObject):
    """
    Class implementing the Django project plugin.
    """
    lexerAssociations = {
        "*.html" : "Django", 
        "*.htm"  : "Django", 
    }
    
    def __init__(self, ui):
        """
        Constructor
        
        @param ui reference to the user interface object (UI.UserInterface)
        """
        QObject.__init__(self, ui)
        self.__ui = ui
        self.__initialize()
        
        self.__defaults = {
            "ServerAddress"  : "", 
            
            "RecentNumberApps" : 10, 
            "UsePlainPython" : 0, 
        }
        if sys.platform == "win32":
            self.__defaults["ConsoleCommandNoClose"] = "cmd.exe /k"
            self.__defaults["ConsoleCommand"] = "cmd.exe /c"
        else:
            self.__defaults["ConsoleCommandNoClose"] = "konsole --noclose -e"
            self.__defaults["ConsoleCommand"] = "konsole -e"
        
        self.__translator = None
        self.__loadTranslator()
    
    def __initialize(self):
        """
        Private slot to (re)initialize the plugin.
        """
        self.__object = None
        
        self.__mainAct = None
        self.__mainMenu = None
        
        self.__e4project = e4App().getObject("Project")
    
    def __checkVersions(self):
        """
        Private function to check that the eric4 version is ok.
        
        @return flag indicating version is ok (boolean)
        """
        global error
        
        if self.__ui.versionIsNewer('4.1.99', '20080731'):
            error = QString("")
        else:
            error = self.trUtf8("eric4 version is too old, %1, %2 or newer needed.")\
                .arg("4.2.0").arg("20080801")
            return False
        
        return True
    
    def activate(self):
        """
        Public method to activate this plugin.
        
        @return tuple of None and activation status (boolean)
        """
        if not self.__checkVersions():
            return None, False
        
        global djangoPluginObject
        djangoPluginObject = self
        
        self.__object = Project(self, self.__ui)
        self.__object.initActions()
        e4App().registerPluginObject("ProjectDjango", self.__object)
        
        self.__mainMenu = self.__object.initMenu()
        
        try:
            self.__e4project.registerProjectType("Django", self.trUtf8("Django"), 
                self.fileTypesCallback, 
                lexerAssociationCallback = self.lexerAssociationCallback)
        except TypeError:
            self.__e4project.registerProjectType("Django", self.trUtf8("Django"), 
                self.fileTypesCallback)
        Preferences.setProjectBrowserFlagsDefault("Django", 
            SourcesBrowserFlag | \
            FormsBrowserFlag | \
            TranslationsBrowserFlag | \
            OthersBrowserFlag,
        )
        
        if self.__e4project.isOpen():
            self.__projectOpened()
            self.__object.projectOpenedHooks()
        
        if self.__ui.versionIsNewer('4.1.99', '20080727'):
            self.connect(e4App().getObject("Project"), SIGNAL('projectOpened'),
                         self.__projectOpened)
            self.connect(e4App().getObject("Project"), SIGNAL('projectClosed'),
                         self.__projectClosed)
            self.connect(e4App().getObject("Project"), SIGNAL('newProject'),
                         self.__projectOpened)
            
            self.connect(e4App().getObject("Project"), SIGNAL('projectAboutToBeCreated'), 
                         self.__object.startProjectOrApplication)
        
        self.connect(e4App().getObject("Project"), SIGNAL('projectOpenedHooks'),
                     self.__object.projectOpenedHooks)
        self.connect(e4App().getObject("Project"), SIGNAL('projectClosedHooks'),
                     self.__object.projectClosedHooks)
        self.connect(e4App().getObject("Project"), SIGNAL('newProjectHooks'),
                     self.__object.projectOpenedHooks)
        
        if self.__ui.versionIsNewer('4.2.99', '20081118'):
            import QScintilla.Lexers
            QScintilla.Lexers.registerLexer(
                "Django", 
                self.trUtf8("Django"), 
                "dummy.django", 
                self.getLexer, 
                QStringList(self.trUtf8('Django Template Files (*.django)')), 
                QStringList(self.trUtf8('Django Template Files (*.django)')), 
                ['*.django']
            )
        
        return None, True
    
    def deactivate(self):
        """
        Public method to deactivate this plugin.
        """
        e4App().unregisterPluginObject("ProjectDjango")
        
        if self.__ui.versionIsNewer('4.2.99', '20081118'):
            import QScintilla.Lexers
            QScintilla.Lexers.unregisterLexer("Django")
        
        if self.__ui.versionIsNewer('4.1.99', '20080727'):
            self.disconnect(e4App().getObject("Project"), SIGNAL('projectOpened'),
                            self.__projectOpened)
            self.disconnect(e4App().getObject("Project"), SIGNAL('projectClosed'),
                            self.__projectClosed)
            self.disconnect(e4App().getObject("Project"), SIGNAL('newProject'),
                            self.__projectOpened)
            
            self.disconnect(e4App().getObject("Project"), 
                            SIGNAL('projectAboutToBeCreated'), 
                            self.__object.startProjectOrApplication)
        
        self.disconnect(e4App().getObject("Project"), SIGNAL('projectOpenedHooks'),
                        self.__object.projectOpenedHooks)
        self.disconnect(e4App().getObject("Project"), SIGNAL('projectClosedHooks'),
                        self.__object.projectClosedHooks)
        self.disconnect(e4App().getObject("Project"), SIGNAL('newProjectHooks'),
                        self.__object.projectOpenedHooks)
        
        self.__e4project.unregisterProjectType("Django")
        
        self.__object.projectClosedHooks()
        self.__projectClosed()
        
        self.__initialize()
    
    def __loadTranslator(self):
        """
        Private method to load the translation file.
        """
        if self.__ui is not None:
            loc = self.__ui.getLocale()
            if loc and loc != "C":
                locale_dir = \
                    os.path.join(os.path.dirname(__file__), "ProjectDjango", "i18n")
                translation = "django_%s" % loc
                translator = QTranslator(None)
                loaded = translator.load(translation, locale_dir)
                if loaded:
                    self.__translator = translator
                    e4App().installTranslator(self.__translator)
                else:
                    print "Warning: translation file '%s' could not be loaded." % \
                        translation
                    print "Using default."
    
    def __projectOpened(self):
        """
        Public slot to handle the projectOpened signal.
        """
        if self.__e4project.getProjectType() == "Django":
            projectAct = self.__ui.getMenuBarAction("project")
            actions = self.__ui.menuBar().actions()
            insertAct = actions[actions.index(projectAct) + 1]
            self.__mainAct = self.__ui.menuBar().insertMenu(insertAct, self.__mainMenu)
    
    def __projectClosed(self):
        """
        Public slot to handle the projectClosed signal.
        """
        if self.__mainAct is not None:
            self.__ui.menuBar().removeAction(self.__mainAct)
            self.__mainAct = None
            self.__object.projectClosed()
    
    def fileTypesCallback(self):
        """
        Public method to get the filetype associations of the Django project type.
        
        @return dictionary with file type associations
        """
        if self.__e4project.getProjectType() == "Django":
            fileTypes = {
                "*.html" : "FORMS", 
                "*.htm"  : "FORMS", 
                "*.js"   : "FORMS", 
                "*.po"   : "TRANSLATIONS", 
                "*.mo"   : "TRANSLATIONS", 
            }
        else:
            fileTypes = {}
        return fileTypes
    
    def lexerAssociationCallback(self, filename):
        """
        Public method to get the lexer association of the Django project type for
        a file.
        
        @param filename name of the file (string or QString)
        @return name of the lexer (string) (Pygments lexers are prefixed with 'Pygments|')
        """
        filename = unicode(filename)
        for pattern, language in self.lexerAssociations.items():
            if fnmatch.fnmatch(filename, pattern):
                return unicode(language)
        return ""
    
    def getPreferences(self, key):
        """
        Public method to retrieve the various refactoring settings.
        
        @param key the key of the value to get
        @param prefClass preferences class used as the storage area
        @return the requested refactoring setting
        """
        if key in ["ConsoleCommand", "ConsoleCommandNoClose", 
                   "ServerAddress"]:
            return Preferences.Prefs.settings.value("Django/" + key,
                QVariant(self.__defaults[key])).toString()
        else:
            return Preferences.Prefs.settings.value("Django/" + key,
                QVariant(self.__defaults[key])).toInt()[0]
    
    def setPreferences(self, key, value):
        """
        Public method to store the various refactoring settings.
        
        @param key the key of the setting to be set (string)
        @param value the value to be set
        @param prefClass preferences class used as the storage area
        """
        Preferences.Prefs.settings.setValue("Django/" + key, QVariant(value))
    
    def prepareUninstall(self):
        """
        Public method to prepare for an uninstallation.
        """
        Preferences.removeProjectBrowserFlags("Django")
        Preferences.Prefs.settings.remove("Django")
        Preferences.Prefs.rsettings.remove("Django")
    
    def getLexer(self, parent = None):
        """
        Public method to instantiate a Django lexer object.
        
        @param parent reference to the parent object (QObject)
        @return reference to the instanciated lexer object (QsciLexer)
        """
        try:
            from ProjectDjango.LexerDjango import LexerDjango
            return LexerDjango(parent)
        except ImportError:
            return None
