# -*- coding: utf-8 -*-

# Copyright (c) 2009 Detlev Offenbach <detlev@die-offenbachs.de>
#


"""
Module implementing the Pylons project plugin.
"""

import sys
import os
import glob
import fnmatch

from PyQt4.QtCore import QObject, SIGNAL, QTranslator, QString, QVariant
from PyQt4.QtGui import QApplication

from KdeQt.KQApplication import e4App

from Project.ProjectBrowser import SourcesBrowserFlag, FormsBrowserFlag, \
    ResourcesBrowserFlag, TranslationsBrowserFlag, InterfacesBrowserFlag, \
    OthersBrowserFlag, AllBrowsersFlag

import Preferences

from ProjectPylons.Project import Project

# Start-of-Header
name = "Pylons Project Plugin"
author = "Detlev Offenbach <detlev@die-offenbachs.de>"
autoactivate = True
deactivateable = True
version = "1.0.0"
className = "ProjectPylonsPlugin"
packageName = "ProjectPylons"
shortDescription = "Project support for Pylons projects."
longDescription = """This plugin implements project support for Pylons projects."""
needsRestart = False
# End-of-Header

error = QString("")

pylonsPluginObject = None

def createPylonsPage(configDlg):
    """
    Module function to create the Pylons configuration page.
    
    @return reference to the configuration page
    """
    global pylonsPluginObject
    from ProjectPylons.ConfigurationPage.PylonsPage import PylonsPage
    page = PylonsPage(pylonsPluginObject)
    return page
    
def getConfigData():
    """
    Module function returning data as required by the configuration dialog.
    
    @return dictionary containing the relevant data
    """
    if e4App().getObject("UserInterface").versionIsNewer('4.1.99', '20080731'):
        return {
            "pylonsPage" : \
                [QApplication.translate("ProjectPylonsPlugin", "Pylons"), 
                 os.path.join("ProjectPylons", "icons", 
                              "pylons.png"),
                 createPylonsPage, None, None],
        }
    else:
        return {}

def apiFiles(language):
    """
    Module function to return the API files made available by this plugin.
    
    @return list of API filenames (list of string)
    """
    if language == "Python":
        apisDir = \
            os.path.join(os.path.dirname(__file__), "ProjectPylons", "APIs")
        apis = glob.glob(os.path.join(apisDir, '*.api'))
    else:
        apis = []
    return apis

def prepareUninstall():
    """
    Module function to prepare for an uninstallation.
    """
    pylons = ProjectPylonsPlugin(None)
    pylons.prepareUninstall()
    
class ProjectPylonsPlugin(QObject):
    """
    Class implementing the Pylons project plugin.
    """
    lexerAssociations = {
        "*.mako" : "Pygments|HTML+Mako", 
        "*.html" : "Pygments|HTML+Mako", 
        "*.htm"  : "Pygments|HTML+Mako", 
    }
    
    def __init__(self, ui):
        """
        Constructor
        
        @param ui reference to the user interface object (UI.UserInterface)
        """
        QObject.__init__(self, ui)
        self.__ui = ui
        self.__initialize()
        
        self.__defaults = {
            "VirtualEnvironment"  : "", 
            "ServerPort" : 0, 
            "UsePlainPython" : 0, 
            "PylonsDocUrl" : "http://docs.pylonshq.com/", 
        }
        if sys.platform == "win32":
            self.__defaults["ConsoleCommandNoClose"] = "cmd.exe /k"
            self.__defaults["ConsoleCommand"] = "cmd.exe /c"
        else:
            self.__defaults["ConsoleCommandNoClose"] = "konsole --noclose -e"
            self.__defaults["ConsoleCommand"] = "konsole -e"
        
        self.__translator = None
        self.__loadTranslator()
    
    def __initialize(self):
        """
        Private slot to (re)initialize the plugin.
        """
        self.__object = None
        
        self.__mainAct = None
        self.__mainMenu = None
        
        self.__e4project = e4App().getObject("Project")
    
    def __checkVersions(self):
        """
        Private function to check that the eric4 version is ok.
        
        @return flag indicating version is ok (boolean)
        """
        global error
        
        if self.__ui.versionIsNewer('4.1.99', '20080229'):
            error = QString("")
        else:
            error = self.trUtf8("eric4 version is too old, %1, %2 or newer needed.")\
                .arg("4.2.0").arg("20080229")
            return False
        
        return True
    
    def activate(self):
        """
        Public method to activate this plugin.
        
        @return tuple of None and activation status (boolean)
        """
        if not self.__checkVersions():
            return None, False
        
        global pylonsPluginObject
        pylonsPluginObject = self
        
        self.__object = Project(self, self.__ui)
        self.__object.initActions()
        e4App().registerPluginObject("ProjectPylons", self.__object)
        
        self.__mainMenu = self.__object.initMenu()
        
        try:
            self.__e4project.registerProjectType("Pylons", self.trUtf8("Pylons"), 
                self.fileTypesCallback, 
                lexerAssociationCallback = self.lexerAssociationCallback, 
                binaryTranslationsCallback = self.binaryTranslationsCallback)
        except TypeError:
            self.__e4project.registerProjectType("Pylons", self.trUtf8("Pylons"), 
                self.fileTypesCallback, 
                binaryTranslationsCallback = self.binaryTranslationsCallback)
        Preferences.setProjectBrowserFlagsDefault("Pylons", 
            SourcesBrowserFlag | \
            FormsBrowserFlag | \
            TranslationsBrowserFlag | \
            OthersBrowserFlag,
        )
        
        if self.__e4project.isOpen():
            self.__projectOpened()
            self.__object.projectOpenedHooks()
        
        self.connect(e4App().getObject("Project"), SIGNAL('projectOpened'),
                     self.__projectOpened)
        self.connect(e4App().getObject("Project"), SIGNAL('projectClosed'),
                     self.__projectClosed)
        self.connect(e4App().getObject("Project"), SIGNAL('newProject'),
                     self.__projectOpened)
        
        self.connect(e4App().getObject("Project"), SIGNAL('projectOpenedHooks'),
                     self.__object.projectOpenedHooks)
        self.connect(e4App().getObject("Project"), SIGNAL('projectClosedHooks'),
                     self.__object.projectClosedHooks)
        self.connect(e4App().getObject("Project"), SIGNAL('newProjectHooks'),
                     self.__object.projectOpenedHooks)
        
        return None, True
    
    def deactivate(self):
        """
        Public method to deactivate this plugin.
        """
        e4App().unregisterPluginObject("ProjectPylons")
        
        self.disconnect(e4App().getObject("Project"), SIGNAL('projectOpened'),
                        self.__projectOpened)
        self.disconnect(e4App().getObject("Project"), SIGNAL('projectClosed'),
                        self.__projectClosed)
        self.disconnect(e4App().getObject("Project"), SIGNAL('newProject'),
                        self.__projectOpened)
        
        self.disconnect(e4App().getObject("Project"), SIGNAL('projectOpenedHooks'),
                        self.__object.projectOpenedHooks)
        self.disconnect(e4App().getObject("Project"), SIGNAL('projectClosedHooks'),
                        self.__object.projectClosedHooks)
        self.disconnect(e4App().getObject("Project"), SIGNAL('newProjectHooks'),
                        self.__object.projectOpenedHooks)
        
        self.__e4project.unregisterProjectType("Pylons")
        
        self.__object.projectClosedHooks()
        self.__projectClosed()
        
        self.__initialize()
    
    def __loadTranslator(self):
        """
        Private method to load the translation file.
        """
        if self.__ui is not None:
            loc = self.__ui.getLocale()
            if loc and loc != "C":
                locale_dir = \
                    os.path.join(os.path.dirname(__file__), "ProjectPylons", "i18n")
                translation = "pylons_%s" % loc
                translator = QTranslator(None)
                loaded = translator.load(translation, locale_dir)
                if loaded:
                    self.__translator = translator
                    e4App().installTranslator(self.__translator)
                else:
                    print "Warning: translation file '%s' could not be loaded." % \
                        translation
                    print "Using default."
    
    def __projectOpened(self):
        """
        Private slot to handle the projectOpened signal.
        """
        if self.__e4project.getProjectType() == "Pylons":
            projectAct = self.__ui.getMenuBarAction("project")
            actions = self.__ui.menuBar().actions()
            insertAct = actions[actions.index(projectAct) + 1]
            self.__mainAct = self.__ui.menuBar().insertMenu(insertAct, self.__mainMenu)
    
    def __projectClosed(self):
        """
        Private slot to handle the projectClosed signal.
        """
        if self.__mainAct is not None:
            self.__ui.menuBar().removeAction(self.__mainAct)
            self.__mainAct = None
            self.__object.projectClosed()
    
    def fileTypesCallback(self):
        """
        Public method get the filetype associations of the TurboGears project type.
        
        @return dictionary with file type associations
        """
        if self.__e4project.getProjectType() == "Pylons":
            fileTypes = {
                "*.mako" : "FORMS", 
                "*.html" : "FORMS", 
                "*.htm"  : "FORMS", 
                "*.js"   : "FORMS",
                "*.pot"  : "TRANSLATIONS", 
                "*.po"   : "TRANSLATIONS", 
                "*.mo"   : "TRANSLATIONS", 
            }
        else:
            fileTypes = {}
        return fileTypes
    
    def lexerAssociationCallback(self, filename):
        """
        Public method to get the lexer association of the Pylons project type for
        a file.
        
        @param filename name of the file (string or QString)
        @return name of the lexer (string) (Pygments lexers are prefixed with 'Pygments|')
        """
        filename = unicode(filename)
        for pattern, language in self.lexerAssociations.items():
            if fnmatch.fnmatch(filename, pattern):
                return unicode(language)
        return ""
    
    def binaryTranslationsCallback(self, filename):
        """
        Public method to determine the filename of a compiled translation file
        given the translation source file.
        
        @param filename name of the translation source file (string or QString)
        @return name of the binary translation file (string)
        """
        fn = unicode(filename)
        if fn.endswith(".po"):
            fn = fn.replace(".po", ".mo")
        return fn
    
    def getPreferences(self, key):
        """
        Public method to retrieve the various refactoring settings.
        
        @param key the key of the value to get
        @param prefClass preferences class used as the storage area
        @return the requested refactoring setting
        """
        if key in ["ConsoleCommand", "ConsoleCommandNoClose", 
                   "VirtualEnvironment", "PylonsDocUrl"]:
            return Preferences.Prefs.settings.value("Pylons/" + key,
                QVariant(self.__defaults[key])).toString()
        else:
            return Preferences.Prefs.settings.value("Pylons/" + key,
                QVariant(self.__defaults[key])).toInt()[0]
    
    def setPreferences(self, key, value):
        """
        Public method to store the various refactoring settings.
        
        @param key the key of the setting to be set (string)
        @param value the value to be set
        @param prefClass preferences class used as the storage area
        """
        Preferences.Prefs.settings.setValue("Pylons/" + key, QVariant(value))
    
    def prepareUninstall(self):
        """
        Public method to prepare for an uninstallation.
        """
        Preferences.removeProjectBrowserFlags("Pylons")
        Preferences.Prefs.settings.remove("Pylons")
