# -*- coding: utf-8 -*-

# Copyright (c) 2007 - 2009 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the CVS version control plugin.
"""

import os
import sys

from PyQt4.QtCore import QString, QObject, SIGNAL, QTranslator, QVariant
from PyQt4.QtGui import QApplication

from KdeQt import KQMessageBox
from KdeQt.KQApplication import e4App

from E4Gui.E4Action import E4Action

from vcsCVS.cvs import Cvs

import Preferences
from Preferences.Shortcuts import readShortcuts

import UI.PixmapCache

# Start-of-Header
name = "CVS Plugin"
author = "Detlev Offenbach <detlev@die-offenbachs.de>"
autoactivate = False
deactivateable = True
version = "4.1.12"
pluginType = "version_control"
pluginTypename = "CVS"
displayString = QApplication.translate('VcsCVSPlugin', 'CVS')
className = "VcsCVSPlugin"
packageName = "vcsCVS"
shortDescription = "Implements the CVS version control interface."
longDescription = """This plugin provides the CVS version control interface."""
needsRestart = False
# End-of-Header

error = QString("")

def exeDisplayData():
    """
    Public method to support the display of some executable info.
    
    @return dictionary containing the data to query the presence of
        the executable
    """
    exe = 'cvs'
    if sys.platform == "win32":
        exe += '.exe'
    
    data = {
        "programEntry"      : True, 
        "header"            : QApplication.translate("VcsCVSPlugin",
                                "Version Control - CVS"), 
        "exe"               : exe, 
        "versionCommand"    : '--version', 
        "versionStartsWith" : 'Concurrent Versions', 
        "versionPosition"   : 4, 
        "version"           : "", 
        "versionCleanup"    : None, 
    }
    
    return data

def getVcsSystemIndicator():
    """
    Public function to get the indicators for this version control system.
    
    @return dictionary with indicator as key and a tuple with the vcs name (string)
        and vcs display string (QString)
    """
    global displayString, pluginTypename
    data = {}
    data["CVS"] = (pluginTypename, displayString)
    return data

__cvsAdminObject = None
__translator = None
__cvsCfgPluginObject = None

def moduleSetup():
    """
    Public function to do some module level setup.
    """
    global __cvsAdminObject
    __cvsAdminObject = CVSAdminObject()
    __loadTranslator()

def __loadTranslator():
    """
    Private function to load the translation file.
    """
    global __translator
    loc = e4App().getObject("UserInterface").getLocale()
    if loc and loc != "C":
        locale_dir = \
            os.path.join(os.path.dirname(__file__), "vcsCVS", "i18n")
        translation = "cvs_%s" % loc
        translator = QTranslator(None)
        loaded = translator.load(translation, locale_dir)
        if loaded:
            __translator = translator
            e4App().installTranslator(__translator)
        else:
            print "Warning: translation file '%s' could not be loaded." % \
                translation
            print "Using default."

def createConfigurationPage(configDlg):
    """
    Module function to create the configuration page.
    
    @return reference to the configuration page
    """
    global __cvsCfgPluginObject
    from vcsCVS.ConfigurationPage.CvsPage import CvsPage
    if __cvsCfgPluginObject is None:
        __cvsCfgPluginObject = VcsCVSPlugin(None)
    page = CvsPage(__cvsCfgPluginObject)
    return page
    
def getConfigData():
    """
    Module function returning data as required by the configuration dialog.
    
    @return dictionary with key "zzz_cvsPage" containing the relevant data
    """
    return {
        "zzz_cvsPage" : \
            [QApplication.translate("VcsCVSPlugin", "CVS"), 
             "preferences-vcs.png",
             createConfigurationPage, "vcsPage", None],
    }

def prepareUninstall():
    """
    Module function to prepare for an uninstallation.
    """
    Preferences.Prefs.settings.remove("CVS")
    
class CVSAdminObject(QObject):
    """
    Class implementing an object to realize the CVS admin entries of the
    generic VCS interface.
    """
    def __init__(self):
        QObject.__init__(self)
        
        self.cvsLoginAct = E4Action(self.trUtf8('Login to CVS repository'),
                UI.PixmapCache.getIcon("vcsLogin.png"),
                self.trUtf8('Log&in to CVS repository...'), 0, 0, self, 'vcs_cvslogin')
        self.cvsLoginAct.setStatusTip(self.trUtf8(
            'Login to CVS repository'
        ))
        self.cvsLoginAct.setWhatsThis(self.trUtf8(
            """<b>Login to CVS repository</b>"""
            """<p>This performs a login to the CVS repository.</p>"""
        ))
        self.connect(self.cvsLoginAct, SIGNAL('triggered()'), self.__cvsLogin)
        
        self.cvsLogoutAct = E4Action(self.trUtf8('Logout from CVS repository'),
                UI.PixmapCache.getIcon("vcsLogout.png"),
                self.trUtf8('Log&out from CVS repository...'), 
                0, 0, self, 'vcs_cvslogout')
        self.cvsLogoutAct.setStatusTip(self.trUtf8(
            'Logout from CVS repository'
        ))
        self.cvsLogoutAct.setWhatsThis(self.trUtf8(
            """<b>Logout from CVS repository</b>"""
            """<p>This performs a logout from the CVS repository.</p>"""
        ))
        self.connect(self.cvsLogoutAct, SIGNAL('triggered()'), self.__cvsLogout)
        
        vcs = Cvs(None)
        self.cvsExists, msg = vcs.vcsExists()
        self.connect(e4App().getObject("Project"), SIGNAL("showMenu"), self.__showVcsMenu)
    
    def __showVcsMenu(self, menuName, menu):
        """
        Private slot to handle the project showMenu signal.
        """
        if menuName == "VCS" and self.cvsExists:
            if not self.cvsLoginAct in menu.actions():
                menu.addAction(self.cvsLoginAct)
                menu.addAction(self.cvsLogoutAct)
    
    def __cvsLogin(self):
        """
        Private slot used to login to a remote repository.
        """
        vcs = Cvs(None)
        vcs.cvsLogin()
    
    def __cvsLogout(self):
        """
        Private slot used to logout from a remote repository.
        """
        vcs = Cvs(None)
        vcs.cvsLogout()

class VcsCVSPlugin(object):
    """
    Class implementing the CVS version control plugin.
    """
    def __init__(self, ui):
        """
        Constructor
        
        @param ui reference to the user interface object (UI.UserInterface)
        """
        self.__ui = ui
        
        self.__cvsDefaults = {
            "CommitMessages" : 20, 
        }
        
        if self.__ui and self.__ui.versionIsNewer('4.2.99', '20081109'):
            from vcsCVS.ProjectHelper import CvsProjectHelper
            self.__projectHelperObject = CvsProjectHelper(None, None)
            e4App().registerPluginObject(pluginTypename, self.__projectHelperObject, 
                                         pluginType)
            readShortcuts(pluginName = pluginTypename)

    def getProjectHelper(self):
        """
        Public method to get a reference to the project helper object.
        
        @return reference to the project helper object
        """
        return self.__projectHelperObject
    
    def activate(self):
        """
        Public method to activate this plugin.
        
        @return tuple of reference to instantiated viewmanager and 
            activation status (boolean)
        """
        self.__object = Cvs(self, self.__ui)
        return self.__object, True
    
    def deactivate(self):
        """
        Public method to deactivate this plugin.
        """
        self.__object = None
    
    def getPreferences(self, key):
        """
        Public method to retrieve the various refactoring settings.
        
        @param key the key of the value to get
        @param prefClass preferences class used as the storage area
        @return the requested refactoring setting
        """
        if key in ["Commits"]:
            return Preferences.Prefs.settings.value("CVS/" + key).toStringList()
        else:
            return Preferences.Prefs.settings.value("CVS/" + key,
                QVariant(self.__cvsDefaults[key])).toInt()[0]
    
    def setPreferences(self, key, value):
        """
        Public method to store the various refactoring settings.
        
        @param key the key of the setting to be set
        @param value the value to be set
        @param prefClass preferences class used as the storage area
        """
        Preferences.Prefs.settings.setValue("CVS/" + key, QVariant(value))
    
    def prepareUninstall(self):
        """
        Public method to prepare for an uninstallation.
        """
        e4App().unregisterPluginObject(pluginTypename)
