# -*- coding: utf-8 -*-

# Copyright (c) 2008 - 2009 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a custom lexer for Django templates.
"""

import sys

from pygments.token import Token
from pygments.lexers import get_lexer_by_name

from PyQt4.QtCore import QString
from PyQt4.QtGui import QColor, QFont

from QScintilla.Lexers.LexerContainer import LexerContainer

DJANGO_DEFAULT, \
DJANGO_COMMENT, \
DJANGO_NUMBER, \
DJANGO_STRING, \
DJANGO_STRINGEOL, \
DJANGO_SCALAR, \
DJANGO_OPERATOR, \
DJANGO_PREPROCESSOR, \
DJANGO_ATTRIBUTE, \
DJANGO_TAG, \
DJANGO_BUILTIN, \
DJANGO_KEYWORD, \
DJANGO_FUNCTION, \
DJANGO_VARIABLE         = range(14)

#-----------------------------------------------------------------------------#

TOKEN_MAP = { Token.Literal.String  : DJANGO_STRING,
              Token.Comment.Preproc : DJANGO_PREPROCESSOR,
              Token.Comment         : DJANGO_COMMENT,
              Token.Name.Builtin    : DJANGO_BUILTIN,
              Token.Operator        : DJANGO_OPERATOR,
              Token.Punctuation     : DJANGO_OPERATOR,
              Token.Number          : DJANGO_NUMBER,
              Token.Keyword         : DJANGO_KEYWORD,
              Token.Name.Attribute  : DJANGO_ATTRIBUTE,
              Token.String.Interpol : DJANGO_SCALAR,
              Token.Name.Tag        : DJANGO_TAG, 
              Token.Name.Function   : DJANGO_FUNCTION, 
              Token.Name.Variable   : DJANGO_VARIABLE, 
            }

#-----------------------------------------------------------------------------#

class LexerDjango(LexerContainer):
    """ 
    Class implementing a custom lexer for Django templates.
    """
    def __init__(self, parent=None):
        """
        Constructor
        
        @param parent parent widget of this lexer
        """
        LexerContainer.__init__(self, parent)
        
        self.streamCommentString = {
            'start' : QString('<!-- '),
            'end'   : QString(' -->')
        }
        
        self.__lexer = get_lexer_by_name("html+django")
        
        self.descriptions = [
            self.trUtf8("Default"), 
            self.trUtf8("Comment"), 
            self.trUtf8("Number"), 
            self.trUtf8("String"), 
            self.trUtf8("Unclosed String"), 
            self.trUtf8("Scalar"),
            self.trUtf8("Operator"), 
            self.trUtf8("Preprocessor"), 
            self.trUtf8("Attribute"), 
            self.trUtf8("Tag"), 
            self.trUtf8("Builtin"), 
            self.trUtf8("Keyword"), 
            self.trUtf8("Function or method name"), 
            self.trUtf8("Identifier"), 
        ]
        
        self.defaultColors = [
            QColor(0x80, 0x80, 0x80),     # default
            QColor(0x00, 0x7f, 0x00),     # comment
            QColor(0x00, 0x7f, 0x7f),     # number
            QColor(0x7f, 0x00, 0x7f),     # string
            QColor(0x00, 0x00, 0x00),     # unclosed string
            QColor(0x00, 0x00, 0x00),     # scalar
            QColor(0x00, 0x00, 0x00),     # operator
            QColor(0x7f, 0x7f, 0x00),     # preprocessor
            QColor(0x00, 0x80, 0x80),     # attribute
            QColor(0x00, 0x00, 0x80),     # tag
            QColor(0x00, 0x00, 0xcf),     # builtin
            QColor(0x7f, 0x00, 0x00),     # keyword
            QColor(0x00, 0x7f, 0x7f),     # function
            QColor(0x00, 0x00, 0x00),     # variable
        ]
        
        self.keywords = ("true false undefined null in as reversed recursive not and or "
                         "is if else import with loop block forloop")
    
    def language(self):
        """
        Public method returning the language of the lexer.
        
        @return language of the lexer (string)
        """
        return "Django"
    
    def description(self, style):
        """
        Public method returning the descriptions of the styles supported
        by the lexer.
        
        @param style style number (integer)
        """
        try:
            return self.descriptions[style]
        except IndexError:
            return QString()
    
    def defaultColor(self, style):
        """
        Public method to get the default foreground color for a style.
        
        @param style style number (integer)
        @return foreground color (QColor)
        """
        try:
            return self.defaultColors[style]
        except IndexError:
            return LexerContainer.defaultColor(self, style)
    
    def defaultPaper(self, style):
        """
        Public method to get the default background color for a style.
        
        @param style style number (integer)
        @return background color (QColor)
        """
        if style == DJANGO_STRINGEOL:
            return QColor(0xe0, 0xc0, 0xe0)
        
        return LexerContainer.defaultPaper(self, style)
    
    def defaultEolFill(self, style):
        """
        Public method to get the default eolfill setting for a style.
        
        @param style style number (integer)
        @return eolfill setting (boolean)
        """
        if style == DJANGO_STRINGEOL:
            return True
        
        return LexerContainer.defaultEolFill(self, style)
    
    def defaultFont(self, style):
        """
        Public method to get the default font for a style.
        
        @param style style number (integer)
        @return font (QFont)
        """
        if style in [DJANGO_COMMENT]:
            if sys.platform == "win32":
                return QFont("Comic Sans MS", 9)
            else:
                return QFont("Bitstream Vera Serif", 9)
        
        if style in [DJANGO_STRING, DJANGO_STRINGEOL]:
            if sys.platform == "win32":
                return QFont("Comic Sans MS", 10)
            else:
                return QFont("Bitstream Vera Serif", 10)
        
        if style in [DJANGO_KEYWORD, DJANGO_OPERATOR, DJANGO_BUILTIN, DJANGO_ATTRIBUTE, 
                     DJANGO_FUNCTION]:
            f = LexerContainer.defaultFont(self, style)
            f.setBold(True)
            return f
        
        return LexerContainer.defaultFont(self, style)
    
    def keywords(self, set):
        """
        Public method to get the list of keywords.
        
        @param set number of keyword set (integer, one based)
        @return list of supported keywords (string) or None
        """
        if set == 1:
            return self.keywords
        else:
            return None
    
    def styleText(self, start, end):
        """
        Public method to perform the styling.
        
        @param start position of first character to be styled (integer)
        @param end position of last character to be styled (integer)
        """
        eolLen = len(self.editor.getLineSeparator())
        cpos = 0
        self.editor.startStyling(cpos, 0x1f)
        for token, txt in self.__lexer.get_tokens(unicode(self.editor.text())[:end + 1]):
            style = TOKEN_MAP.get(token, DJANGO_DEFAULT)
            if style == DJANGO_PREPROCESSOR and txt.startswith(u'#'):
                style = DJANGO_COMMENT
            elif style == DJANGO_STRING:
                if txt and txt[0] in '"\''and txt[-1] not in '"\'':
                    style = DJANGO_STRINGEOL

            tlen = len(txt)
            if eolLen > 1:
                tlen += txt.count('\n')
            if tlen:
                self.editor.setStyling(tlen, style)
            cpos += tlen
            self.editor.startStyling(cpos, 0x1f)
