# -*- coding: utf-8 -*-

# Copyright (c) 2009 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the Pylons project support.
"""

import os
import sys
import re

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from KdeQt.KQApplication import e4App
from KdeQt import KQFileDialog, KQMessageBox, KQInputDialog

from E4Gui.E4Action import E4Action

from FormSelectionDialog import FormSelectionDialog
from CreateParametersDialog import CreateParametersDialog
from CreateRestControllerParametersDialog import CreateRestControllerParametersDialog
from PylonsDialog import PylonsDialog

import Utilities

class PylonsNoSiteSelectedException(Exception):
    """
    Exception thrown to signal, that there is no current site.
    """
    pass

class Project(QObject):
    """
    Class implementing the Pylons project support.
    """
    def __init__(self, plugin, parent = None):
        """
        Constructor
        
        @param plugin reference to the plugin object
        @param parent parent (QObject)
        """
        QObject.__init__(self, parent)
        
        self.__plugin = plugin
        self.__ui = parent
        self.__e4project = e4App().getObject("Project")
        self.__hooksInstalled = False
        
        self.__mainMenu = None
        
        self.serverProc = None
    
    def initActions(self):
        """
        Public method to define the Pylons actions.
        """
        self.actions = []
    
        self.selectSiteAct = E4Action(self.trUtf8('Current Site'),
                QString(),
                0, 0,
                self,'pylons_current_site')
        self.selectSiteAct.setStatusTip(self.trUtf8(
            'Selects the current site'))
        self.selectSiteAct.setWhatsThis(self.trUtf8(
            """<b>Current Site</b>"""
            """<p>Selects the current site. Used for multi-site """
            """Pylons projects to switch between the sites.</p>"""
        ))
        self.connect(self.selectSiteAct, SIGNAL('triggered()'), self.__selectSite)
        self.actions.append(self.selectSiteAct)
        self.__setCurrentSite(None)
        
        ###############################
        ## create actions below      ##
        ###############################
        
        self.createSiteAct = E4Action(self.trUtf8('Create Site'),
                self.trUtf8('Create Si&te'), 
                0, 0,
                self,'pylons_create_site')
        self.createSiteAct.setStatusTip(self.trUtf8(
            'Creates a new Pylons site'))
        self.createSiteAct.setWhatsThis(self.trUtf8(
            """<b>Create Site</b>"""
            """<p>Creates a new Pylons site using "paster create".</p>"""
        ))
        self.connect(self.createSiteAct, SIGNAL('triggered()'), self.__createSite)
        self.actions.append(self.createSiteAct)
        
        self.createControllerAct = E4Action(self.trUtf8('Create Controller'),
                self.trUtf8('Create &Controller'), 
                0, 0,
                self,'pylons_create_controller')
        self.createControllerAct.setStatusTip(self.trUtf8(
            'Creates a new controller for the current site'))
        self.createControllerAct.setWhatsThis(self.trUtf8(
            """<b>Create Controller</b>"""
            """<p>Creates a new controller for the current site using"""
            """ "paster controller".</p>"""
        ))
        self.connect(self.createControllerAct, SIGNAL('triggered()'), 
            self.__createController)
        self.actions.append(self.createControllerAct)
        
        self.createRestControllerAct = E4Action(self.trUtf8('Create REST Controller'),
                self.trUtf8('Create &REST Controller'), 
                0, 0,
                self,'pylons_create_restcontroller')
        self.createRestControllerAct.setStatusTip(self.trUtf8(
            'Creates a new REST controller for the current site'))
        self.createRestControllerAct.setWhatsThis(self.trUtf8(
            """<b>Create REST Controller</b>"""
            """<p>Creates a new REST controller for the current site using"""
            """ "paster restcontroller".</p>"""
        ))
        self.connect(self.createRestControllerAct, SIGNAL('triggered()'), 
            self.__createRestController)
        self.actions.append(self.createRestControllerAct)
        
        ##############################
        ## run actions below        ##
        ##############################
        
        self.runServerAct = E4Action(self.trUtf8('Run Server'),
                self.trUtf8('Run &Server'), 
                0, 0,
                self,'pylons_run_server')
        self.runServerAct.setStatusTip(self.trUtf8(
            'Starts the Pylons Web server'))
        self.runServerAct.setWhatsThis(self.trUtf8(
            """<b>Run Server</b>"""
            """<p>Starts the Pylons Web server using"""
            """ "paster serve --reload development.ini".</p>"""
        ))
        self.connect(self.runServerAct, SIGNAL('triggered()'), self.__runServer)
        self.actions.append(self.runServerAct)
        
        self.runLoggingServerAct = E4Action(self.trUtf8('Run Server with Logging'),
                self.trUtf8('Run Server with &Logging'), 
                0, 0,
                self,'pylons_run_logging_server')
        self.runLoggingServerAct.setStatusTip(self.trUtf8(
            'Starts the Pylons Web server with logging'))
        self.runLoggingServerAct.setWhatsThis(self.trUtf8(
            """<b>Run Server with Logging</b>"""
            """<p>Starts the Pylons Web server with logging using"""
            """ "paster serve --log-file=server.log --reload development.ini".</p>"""
        ))
        self.connect(self.runLoggingServerAct, SIGNAL('triggered()'), 
            self.__runLoggingServer)
        self.actions.append(self.runLoggingServerAct)
        
        self.runBrowserAct = E4Action(self.trUtf8('Run Web-Browser'),
                self.trUtf8('Run &Web-Browser'), 
                0, 0,
                self,'pylons_run_browser')
        self.runBrowserAct.setStatusTip(self.trUtf8(
            'Starts the default Web-Browser with the URL of the Pylons Web server'))
        self.runBrowserAct.setWhatsThis(self.trUtf8(
            """<b>Run Web-Browser</b>"""
            """<p>Starts the default Web-Browser with the URL of the """
            """Pylons Web server.</p>"""
        ))
        self.connect(self.runBrowserAct, SIGNAL('triggered()'), self.__runBrowser)
        self.actions.append(self.runBrowserAct)
    
        self.runPythonShellAct = E4Action(self.trUtf8('Start Python Console'),
                self.trUtf8('Start &Python Console'), 
                0, 0,
                self,'pylons_python_console')
        self.runPythonShellAct.setStatusTip(self.trUtf8(
            'Starts a Python interactive interpreter'))
        self.runPythonShellAct.setWhatsThis(self.trUtf8(
            """<b>Start Python Console</b>"""
            """<p>Starts a Python interactive interpreter.</p>"""
        ))
        self.connect(self.runPythonShellAct, SIGNAL('triggered()'), 
            self.__runPythonShell)
        self.actions.append(self.runPythonShellAct)
        
        ##############################
        ## setup actions below      ##
        ##############################
        
        self.setupDevelopAct = E4Action(self.trUtf8('Setup Development Environment'),
                self.trUtf8('Setup &Development Environment'), 
                0, 0,
                self,'pylons_setup_development')
        self.setupDevelopAct.setStatusTip(self.trUtf8(
            'Setup the Pylons site in development mode'))
        self.setupDevelopAct.setWhatsThis(self.trUtf8(
            """<b>Setup Development Environment</b>"""
            """<p>Setup the Pylons site in development mode using"""
            """ "python setup.py develop".</p>"""
        ))
        self.connect(self.setupDevelopAct, SIGNAL('triggered()'), 
            self.__setupDevelop)
        self.actions.append(self.setupDevelopAct)
        
        self.setupApplicationAct = E4Action(self.trUtf8('Setup Application'),
                self.trUtf8('Setup &Application'), 
                0, 0,
                self,'pylons_setup_application')
        self.setupApplicationAct.setStatusTip(self.trUtf8(
            'Setup the Pylons application'))
        self.setupApplicationAct.setWhatsThis(self.trUtf8(
            """<b>Setup Application</b>"""
            """<p>Setup the Pylons application using "paster setup-app".</p>"""
        ))
        self.connect(self.setupApplicationAct, SIGNAL('triggered()'), 
            self.__setupApplication)
        self.actions.append(self.setupApplicationAct)
        
        ##################################
        ## distribution actions below   ##
        ##################################
        
        self.buildEggAct = E4Action(self.trUtf8('Build Egg'),
                self.trUtf8('Build &Egg'), 
                0, 0,
                self,'pylons_build_egg')
        self.buildEggAct.setStatusTip(self.trUtf8(
            'Builds a Python egg for the site'))
        self.buildEggAct.setWhatsThis(self.trUtf8(
            """<b>Build Egg</b>"""
            """<p>Builds a Python egg for the site using"""
            """ "python setup.py bdist_egg".</p>"""
        ))
        self.connect(self.buildEggAct, SIGNAL('triggered()'), 
            self.__buildEgg)
        self.actions.append(self.buildEggAct)
        
        ##################################
        ## documentation action below   ##
        ##################################
        
        self.documentationAct = E4Action(self.trUtf8('Documentation'),
                self.trUtf8('D&ocumentation'), 
                0, 0,
                self,'pylons_documentation')
        self.documentationAct.setStatusTip(self.trUtf8(
            'Shows the help viewer with the Pylons documentation'))
        self.documentationAct.setWhatsThis(self.trUtf8(
            """<b>Documentation</b>"""
            """<p>Shows the help viewer with the Pylons documentation.</p>"""
        ))
        self.connect(self.documentationAct, SIGNAL('triggered()'), 
            self.__showDocumentation)
        self.actions.append(self.documentationAct)
        
        ##############################
        ## about action below       ##
        ##############################
        
        self.aboutPylonsAct = E4Action(self.trUtf8('About Pylons'),
                self.trUtf8('About P&ylons'), 
                0, 0,
                self,'pylons_about')
        self.aboutPylonsAct.setStatusTip(self.trUtf8(
            'Shows some information about Pylons'))
        self.aboutPylonsAct.setWhatsThis(self.trUtf8(
            """<b>About Pylons</b>"""
            """<p>Shows some information about Pylons.</p>"""
        ))
        self.connect(self.aboutPylonsAct, SIGNAL('triggered()'), self.__pylonsInfo)
        self.actions.append(self.aboutPylonsAct)
        
        ##############################
        ## Sphinx actions below       ##
        ##############################
        
        self.sphinxQuickstartAct = E4Action(self.trUtf8('Initialize'),
                self.trUtf8('&Initialize'), 
                0, 0,
                self,'pylons_sphinx_initialize')
        self.sphinxQuickstartAct.setStatusTip(self.trUtf8(
            'Initializes the documentation using "sphinx-quickstart"'))
        self.sphinxQuickstartAct.setWhatsThis(self.trUtf8(
            """<b>Initialize</b>"""
            """<p>Initializes the documentation using "sphinx-quickstart".</p>"""
        ))
        self.connect(self.sphinxQuickstartAct, SIGNAL('triggered()'), 
            self.__sphinxQuickstart)
        self.actions.append(self.sphinxQuickstartAct)
        
        self.sphinxBuildAct = E4Action(self.trUtf8('Build HTML'),
                self.trUtf8('Build &HTML'), 
                0, 0,
                self,'pylons_sphinx_build_html')
        self.sphinxBuildAct.setStatusTip(self.trUtf8(
            'Builds the HTML documentation using "sphinx-build"'))
        self.sphinxBuildAct.setWhatsThis(self.trUtf8(
            """<b>Build HTML</b>"""
            """<p>Builds the HTML documentation using "sphinx-build".</p>"""
        ))
        self.connect(self.sphinxBuildAct, SIGNAL('triggered()'), 
            self.__sphinxBuildHtml)
        self.actions.append(self.sphinxBuildAct)
    
    def initMenu(self):
        """
        Public slot to initialize the Pylons menu.
        
        @return the menu generated (QMenu)
        """
        sphinxMenu = QMenu(self.trUtf8("Sphin&x"), self.__ui)
        sphinxMenu.setTearOffEnabled(True)
        sphinxMenu.addAction(self.sphinxBuildAct)
        sphinxMenu.addSeparator()
        sphinxMenu.addAction(self.sphinxQuickstartAct)
        
        menu = QMenu(self.trUtf8('P&ylons'), self.__ui)
        menu.setTearOffEnabled(True)
        
        menu.addAction(self.selectSiteAct)
        menu.addSeparator()
        menu.addAction(self.runServerAct)
        menu.addAction(self.runLoggingServerAct)
        menu.addAction(self.runBrowserAct)
        menu.addSeparator()
        menu.addAction(self.createSiteAct)
        menu.addAction(self.createControllerAct)
        menu.addAction(self.createRestControllerAct)
        menu.addSeparator()
        menu.addMenu(sphinxMenu)
        menu.addSeparator()
        menu.addAction(self.setupDevelopAct)
        menu.addAction(self.setupApplicationAct)
        menu.addSeparator()
        menu.addAction(self.runPythonShellAct)
        menu.addSeparator()
        menu.addAction(self.buildEggAct)
        menu.addSeparator()
        menu.addAction(self.documentationAct)
        menu.addSeparator()
        menu.addAction(self.aboutPylonsAct)
        
        self.__mainMenu = menu
        return menu
    
    def projectOpenedHooks(self):
        """
        Public method to add our hook methods.
        """
        if self.__e4project.getProjectType() == "Pylons":
            self.__formsBrowser = \
                e4App().getObject("ProjectBrowser").getProjectBrowser("forms")
            try:
                self.__formsBrowser.addHookMethodAndMenuEntry("newForm", 
                    self.newForm, self.trUtf8("New template..."))
            except AttributeError:
                self.__formsBrowser.addHookMethod("newForm", self.newForm)
            
            if self.__ui.versionIsNewer('4.2.99', '20090127'):
                self.connect(self.__e4project, SIGNAL('projectLanguageAddedByCode'),
                             self.__projectLanguageAdded)
                self.__translationsBrowser = \
                    e4App().getObject("ProjectBrowser").getProjectBrowser("translations")
                self.__translationsBrowser.addHookMethodAndMenuEntry("extractMessages", 
                    self.extractMessages, self.trUtf8("Extract messages"))
                self.__translationsBrowser.addHookMethodAndMenuEntry("releaseAll", 
                    self.compileCatalogs, self.trUtf8("Compile all catalogs"))
                self.__translationsBrowser.addHookMethodAndMenuEntry("releaseSelected", 
                    self.compileSelectedCatalogs, 
                    self.trUtf8("Compile selected catalogs"))
                self.__translationsBrowser.addHookMethodAndMenuEntry("generateAll", 
                    self.updateCatalogs, self.trUtf8("Update all catalogs"))
                self.__translationsBrowser.addHookMethodAndMenuEntry("generateSelected", 
                    self.updateSelectedCatalogs, self.trUtf8("Update selected catalogs"))
            
            self.__hooksInstalled = True
    
    def projectClosedHooks(self):
        """
        Public method to remove our hook methods.
        """
        if self.__hooksInstalled:
            self.__formsBrowser.removeHookMethod("newForm")
            self.__formsBrowser = None
            
            if self.__ui.versionIsNewer('4.2.99', '20090127'):
                self.disconnect(self.__e4project, SIGNAL('projectLanguageAddedByCode'),
                                self.__projectLanguageAdded)
                self.__translationsBrowser.removeHookMethod("extractMessages")
                self.__translationsBrowser.removeHookMethod("releaseAll")
                self.__translationsBrowser.removeHookMethod("releaseSelected")
                self.__translationsBrowser.removeHookMethod("generateAll")
                self.__translationsBrowser.removeHookMethod("generateSelected")
                self.__translationsBrowser = None
            
            self.__hooksInstalled = False
    
    def newForm(self, path):
        """
        Public method to create a new form.
        
        @param path full directory path for the new form file (string)
        """
        dlg = FormSelectionDialog()
        if dlg.exec_() == QDialog.Accepted:
            template = dlg.getTemplateText()
            
            selectedFilter = QString("")
            filter = self.trUtf8(
                "Mako Files (*.mako);;"
                "HTML Files (*.html);;"
                "HTML Files (*.htm);;"
                "All Files (*)")
            fname = KQFileDialog.getSaveFileName(\
                self.__ui,
                self.trUtf8("New Form"),
                path,
                filter,
                selectedFilter,
                QFileDialog.Options(QFileDialog.DontConfirmOverwrite))
            
            if fname.isEmpty():
                # user aborted or didn't enter a filename
                return
            
            ext = QFileInfo(fname).suffix()
            if ext.isEmpty():
                ex = selectedFilter.section('(*', 1, 1).section(')', 0, 0)
                if not ex.isEmpty():
                    fname.append(ex)
            
            fname = unicode(fname)
            if os.path.exists(fname):
                res = KQMessageBox.warning(self,
                    self.trUtf8("New Form"),
                    self.trUtf8("The file already exists! Overwrite it?"),
                    QMessageBox.StandardButtons(\
                        QMessageBox.No | \
                        QMessageBox.Yes),
                    QMessageBox.No)
                if res != QMessageBox.Yes:
                    # user selected to not overwrite
                    return
            
            try:
                f = open(fname, "wb")
                f.write(template)
                f.close()
            except IOError, e:
                KQMessageBox.critical(self,
                    self.trUtf8("New Form"),
                    self.trUtf8("<p>The new form file <b>%1</b> could not be created.<br>"
                        "Problem: %2</p>").arg(fname).arg(unicode(e)))
                return
            
            self.__e4project.appendFile(fname)
            self.__formsBrowser.emit(SIGNAL('sourceFile'), fname)

    ##################################################################
    ## slots below implement general functionality
    ##################################################################
    
    def projectClosed(self):
        """
        Public method to handle the closing of a project.
        """
        if self.serverProc is not None:
            self.__serverProcFinished()
        self.__setCurrentSite(None)
    
    def __getPasterCommand(self):
        """
        Private method to build the paster command.
        
        @return paster command (string)
        """
        paster = "paster"
        if sys.platform == "win32":
            paster += ".exe"
        virtualEnv = unicode(self.__plugin.getPreferences("VirtualEnvironment"))
        if virtualEnv:
            if sys.platform == "win32":
                paster = os.path.join(virtualEnv, "Scripts", paster)
            else:
                paster = os.path.join(virtualEnv, "bin", paster)
        return paster
    
    def __getPythonCommand(self):
        """
        Private method to build the Python command.
        
        @return python command (string)
        """
        python = "python"
        if sys.platform == "win32":
            python += ".exe"
        virtualEnv = unicode(self.__plugin.getPreferences("VirtualEnvironment"))
        if virtualEnv:
            if sys.platform == "win32":
                python = os.path.join(virtualEnv, "Scripts", python)
            else:
                python = os.path.join(virtualEnv, "bin", python)
        return python
    
    def __getSphinxCommand(self, quickstart = False):
        """
        Private method to build the Sphinx command.
        
        @param quickstart flag indicating the quickstart command is requested (boolean)
        @return python command (string)
        """
        sphinx = "sphinx-"
        if quickstart:
            sphinx += "quickstart"
        else:
            sphinx += "build"
        if sys.platform == "win32":
            sphinx += ".exe"
        virtualEnv = unicode(self.__plugin.getPreferences("VirtualEnvironment"))
        if virtualEnv:
            if sys.platform == "win32":
                sphinx = os.path.join(virtualEnv, "Scripts", sphinx)
            else:
                sphinx = os.path.join(virtualEnv, "bin", sphinx)
        return sphinx
    
    def __pylonsInfo(self):
        """
        Private slot to show some info about Pylons.
        """
        version = self.getPylonsVersion()
        url = "http://pylonshq.com"
        KQMessageBox.about(self.__ui,
            self.trUtf8("About Pylons"),
            self.trUtf8(
                "<p>Pylons is a high-level Python Web framework that encourages rapid "
                "development and clean, pragmatic design.</p>"
                "<p><table>"
                "<tr><td>Version:</td><td>%1</td></tr>"
                "<tr><td>URL:</td><td><a href=\"%2\">%2</a></td></tr>"
                "</table></p>"
            ).arg(version).arg(url)
        )
    
    def getPylonsVersion(self):
        """
        Public method to get the Pylons version.
        
        @return Pylons version as a string
        """
        proc = QProcess()
        args = ["-c", "import pylons; print pylons.__version__"]
        proc.start(self.__getPythonCommand(), args)
        started = proc.waitForStarted(3000)
        finished = proc.waitForFinished(3000)
        if not started or not finished:
            return unicode(self.trUtf8("unknown"))
        return unicode(proc.readAllStandardOutput())
    
    def isSpawningConsole(self, consoleCmd):
        """
        Public method to check, if the given console is a spawning console.
        
        @param consoleCmd console command (string)
        @return tuple of two entries giving an indication, if the console
            is spawning (boolean) and the (possibly) cleaned console command
            (string)
        """
        if consoleCmd and consoleCmd[0] == '@':
            return (True, consoleCmd[1:])
        else:
            return (False, consoleCmd)

    ##################################################################
    ## slots below implement creation functions
    ##################################################################
    
    def __createSite(self):
        """
        Private slot to create a new Pylons site.
        """
        dlg = CreateParametersDialog()
        if dlg.exec_() == QDialog.Accepted:
            template, site = dlg.getData()
            
            spawning, consoleCmd = self.isSpawningConsole(
                unicode(self.__plugin.getPreferences("ConsoleCommand")))
            if consoleCmd:
                proc = QProcess()
                args = Utilities.parseOptionString(consoleCmd)
                args[0] = Utilities.getExecutablePath(args[0])
                args.append(self.__getPasterCommand())
                args.append("create")
                args.append("--template=%s" % template)
                args.append(site)
                proc.setWorkingDirectory(self.__e4project.getProjectPath())
                proc.start(args[0], args[1:])
                started = proc.waitForStarted(5000) # wait 5 s
                if not started:
                    KQMessageBox.critical(None,
                        self.trUtf8('Process Generation Error'),
                        self.trUtf8('The Paster process could not be started.'))
                    return
                
                if spawning:
                    KQMessageBox.information(None,
                        self.trUtf8("Spawning Console"),
                        self.trUtf8("""Acknowledge this, when the command run in the"""
                                    """ console has finished."""))
                else:
                    proc.waitForFinished(-1)    # wait until the process finished
                
                # search for files created by paster and add them to the project
                sitePath = os.path.join(self.__e4project.getProjectPath(), site)
                for entry in os.walk(sitePath):
                    for fileName in entry[2]:
                        fullName = os.path.join(entry[0], fileName)
                        self.__e4project.appendFile(fullName)
                
                # create the base directory for translations
                i18nPath = os.path.join(sitePath, site.lower(), "i18n")
                if not os.path.exists(i18nPath):
                    os.makedirs(i18nPath)
                self.__e4project.setDirty(True)
                
                self.__setCurrentSite(site)
    
    def __createController(self):
        """
        Private slot to create a new controller for the current site.
        """
        controller, ok = KQInputDialog.getText(\
            self.__ui,
            self.trUtf8("Create Controller"),
            self.trUtf8("Enter the name of the controller"),
            QLineEdit.Normal)
        if ok and not controller.isEmpty():
            proc = QProcess()
            paster = self.__getPasterCommand()
            args = QStringList("controller")
            args.append(controller)
            proc.setWorkingDirectory(self.__sitePath())
            proc.start(paster, args)
            started = proc.waitForStarted(3000)
            if not started:
                KQMessageBox.critical(None,
                    self.trUtf8('Process Generation Error'),
                    self.trUtf8('The Paster process could not be started.'))
                return
            
            proc.waitForFinished()
            for entry in os.walk(self.__sitePath()):
                for fileName in entry[2]:
                    fullName = os.path.join(entry[0], fileName)
                    if fullName.endswith('.py'):
                        self.__e4project.appendFile(fullName)
    
    def __createRestController(self):
        """
        Private slot to create a new REST controller for the current site.
        """
        dlg = CreateRestControllerParametersDialog()
        if dlg.exec_() == QDialog.Accepted:
            singular, plural = dlg.getData()
            if singular and plural:
                proc = QProcess()
                paster = self.__getPasterCommand()
                args = QStringList("restcontroller")
                args.append(singular)
                args.append(plural)
                proc.setWorkingDirectory(self.__sitePath())
                proc.start(paster, args)
                started = proc.waitForStarted(3000)
                if not started:
                    KQMessageBox.critical(None,
                        self.trUtf8('Process Generation Error'),
                        self.trUtf8('The Paster process could not be started.'))
                    return
                
                proc.waitForFinished()
                for entry in os.walk(self.__sitePath()):
                    for fileName in entry[2]:
                        fullName = os.path.join(entry[0], fileName)
                        if fullName.endswith('.py'):
                            self.__e4project.appendFile(fullName)
    
    ##################################################################
    ## methods below implement site related functions
    ##################################################################
    
    def __findSites(self):
        """
        Private method to determine the relative path of all sites (= top level dirs).
        
        @return list of sites (QStringList)
        """
        sites = QStringList()
        ppath = self.__e4project.getProjectPath()
        for entry in os.listdir(ppath):
            if entry[0] not in "._" and \
               os.path.isdir(os.path.join(ppath, entry)):
                sites.append(entry)
        return sites
    
    def __selectSite(self):
        """
        Private method to select a site to work with.
        
        @return selected site (QString)
        """
        sites = self.__findSites()
        if len(sites) == 0:
            site = None
        elif len(sites) == 1:
            site = sites[0]
        else:
            if self.__currentSite is not None:
                cur = sites.indexOf(self.__currentSite)
                if cur == -1:
                    cur = 0
            else:
                cur = 0
            site, ok = KQInputDialog.getItem(\
                self.__ui,
                self.trUtf8("Select Site"),
                self.trUtf8("Select the Pylons site to work with."),
                sites,
                cur, False)
            if not ok:
                site = None
        self.__setCurrentSite(site)
    
    def __sitePath(self):
        """
        Private method to calculate the full path of the Pylons site.
        
        @exception PylonsNoSiteSelectedException raised, if no site is selected
        @return path of the site (string)
        """
        if self.__currentSite is None:
            self.__selectSite()
        
        if self.__currentSite is None:
            raise PylonsNoSiteSelectedException
        else:
            return os.path.join(self.__e4project.getProjectPath(), 
                                unicode(self.__currentSite))
    
    def __setCurrentSite(self, site):
        """
        Private slot to set the current site.
        
        @param site name of the site (string or QString)
        """
        if site is not None and len(site) == 0:
            self.__currentSite = None
        else:
            self.__currentSite = site
        
        if self.__currentSite is None:
            curSite = self.trUtf8("None")
        else:
            curSite = self.__currentSite
        self.selectSiteAct.setText(
            self.trUtf8('&Current Pylons site (%1)').arg(curSite))
        
        if self.__currentSite is None:
            self.__e4project.pdata["TRANSLATIONPATTERN"] = []
        else:
            usite = unicode(site)
            self.__e4project.pdata["TRANSLATIONPATTERN"] = [
                os.path.join(usite, usite.lower(), "i18n", "%language%", "LC_MESSAGES", 
                             "%s.po" % usite.lower())
            ]
    
    def __site(self):
        """
        Private method to get the name of the current site.
        
        @exception PylonsNoSiteSelectedException raised, if no site is selected
        @return name of the site (string)
        """
        if self.__currentSite is None:
            self.__selectSite()
        
        if self.__currentSite is None:
            raise PylonsNoSiteSelectedException
        else:
            return unicode(self.__currentSite)
    
    ##################################################################
    ## slots below implement run functions
    ##################################################################
    
    def __runServer(self, logging = False):
        """
        Private slot to start the Pylons Web server.
        
        @param logging flag indicating to enable logging (boolean)
        """
        consoleCmd = self.isSpawningConsole(
            unicode(self.__plugin.getPreferences("ConsoleCommand")))[1]
        if consoleCmd:
            args = Utilities.parseOptionString(consoleCmd)
            args[0] = Utilities.getExecutablePath(args[0])
            args.append(self.__getPasterCommand())
            args.append("serve")
            if logging:
                args.append("--log-file=server.log")
            args.append("--reload")
            args.append("development.ini")
            
            try:
                sitePath = self.__sitePath()
                if sys.platform == "win32":
                    serverProcStarted, pid = \
                        QProcess.startDetached(args[0], args[1:], sitePath)
                else:
                    if self.serverProc is not None:
                        self.__serverProcFinished()
                    
                    self.serverProc = QProcess()
                    self.connect(self.serverProc, 
                                 SIGNAL('finished(int, QProcess::ExitStatus)'),
                                 self.__serverProcFinished)
                    self.serverProc.setWorkingDirectory(sitePath)
                    self.serverProc.start(args[0], args[1:])
                    serverProcStarted = self.serverProc.waitForStarted()
                if not serverProcStarted:
                    KQMessageBox.critical(None,
                        self.trUtf8('Process Generation Error'),
                        self.trUtf8('The Pylons server could not be started.'))
            except PylonsNoSiteSelectedException:
                KQMessageBox.warning(None,
                    self.trUtf8('Run Server'),
                    self.trUtf8('No current site selected or no site created yet.'
                                ' Aborting...'))
    
    def __runLoggingServer(self):
        """
        Private slot to start the Pylons Web server with logging.
        """
        self.__runServer(True)
    
    def __serverProcFinished(self):
        """
        Private slot connected to the finished signal.
        """
        if self.serverProc is not None and \
           self.serverProc.state() != QProcess.NotRunning:
            self.serverProc.terminate()
            QTimer.singleShot(2000, self.serverProc, SLOT('kill()'))
            self.serverProc.waitForFinished(3000)
        self.serverProc = None
    
    def __runBrowser(self):
        """
        Private slot to start the default web browser with the server URL.
        """
        port = self.__plugin.getPreferences("ServerPort")
        if port == 0:
            port = QString("5000")
        url = QUrl(QString("http://127.0.0.1:%1").arg(port))
        res = QDesktopServices.openUrl(url)
        if not res:
            KQMessageBox.critical(None,
                self.trUtf8('Run Web-Browser'),
                self.trUtf8('Could not start the web-browser for the URL "%1".')\
                    .arg(url.toString()))
    
    def __runPythonShell(self):
        """
        Private slot to start a Python console for a Pylons site.
        """
        consoleCmd = self.isSpawningConsole(
            unicode(self.__plugin.getPreferences("ConsoleCommand")))[1]
        if consoleCmd:
            proc = QProcess()
            args = Utilities.parseOptionString(consoleCmd)
            args[0] = Utilities.getExecutablePath(args[0])
            args.append(self.__getPasterCommand())
            args.append("shell")
            if self.__plugin.getPreferences("UsePlainPython"):
                args.append("--disable-ipython")
            try:
                wd = self.__sitePath()
                started, pid = QProcess.startDetached(args[0], args[1:], wd)
                if not started:
                    KQMessageBox.critical(None,
                        self.trUtf8('Process Generation Error'),
                        self.trUtf8('The Paster process could not be started.'))
            except PylonsNoSiteSelectedException:
                KQMessageBox.warning(None,
                    self.trUtf8('Start Python Console'),
                    self.trUtf8('No current site selected or no site created yet.'
                                ' Aborting...'))

    ##################################################################
    ## slots below implement setup functions
    ##################################################################
    
    def __setupDevelop(self):
        """
        Private slot to set up the development environment for the current site.
        """
        title = self.trUtf8("Setup Development Environment")
        
        args = QStringList()
        args.append(self.__getPythonCommand())
        args.append("setup.py")
        args.append("develop")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("Pylons development environment setup successfully."))
        res = dia.startProcess(args, wd)
        if res:
            dia.exec_()
    
    def __setupApplication(self):
        """
        Private slot to set up the Pylons application of the current site.
        """
        title = self.trUtf8("Setup Application")
        
        args = QStringList()
        args.append(self.__getPasterCommand())
        args.append("setup-app")
        args.append("development.ini")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("Pylons application setup successfully."))
        res = dia.startProcess(args, wd)
        if res:
            dia.exec_()
    
    ##################################################################
    ## slots below implement distribution functions
    ##################################################################
    
    def __buildEgg(self):
        """
        Private slot to build a Python egg for the current site.
        """
        title = self.trUtf8("Build Egg")
        
        args = QStringList()
        args.append(self.__getPythonCommand())
        args.append("setup.py")
        args.append("bdist_egg")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("Python egg built successfully."))
        res = dia.startProcess(args, wd)
        if res:
            dia.exec_()
    
    ##################################################################
    ## slots below implement documentation functions
    ##################################################################
    
    def __showDocumentation(self):
        """
        Private slot to show the helpviewer with the Pylons documentation.
        """
        page = self.__plugin.getPreferences("PylonsDocUrl")
        self.__ui.launchHelpViewer(page)
    
    ##################################################################
    ## slots below implement translation functions
    ##################################################################
    
    def __getLocale(self, filename):
        """
        Private method to extract the locale out of a file name.
        
        @param filename name of the file used for extraction (string or QString)
        @return extracted locale (string) or None
        """
        if self.__e4project.pdata["TRANSLATIONPATTERN"]:
            pattern = unicode(self.__e4project.pdata["TRANSLATIONPATTERN"][0])\
                .replace("%language%", "(.*?)")
            match = re.search(pattern, unicode(filename))
            if match is not None:
                loc = match.group(1)
                return loc
            else:
                loc = None
        else:
            loc = None
        
        return loc
    
    def __normalizeList(self, filenames):
        """
        Private method to normalize a list of file names.
        
        @param filenames list of file names to normalize (list of string or QString)
        @return normalized file names (list of string)
        """
        nfilenames = []
        for filename in filenames:
            filename = unicode(filename)
            if filename.endswith(".mo"):
                filename = filename.replace(".mo", ".po")
            if filename not in nfilenames:
                nfilenames.append(filename)
        
        return nfilenames
    
    def __siteFilteredList(self, filenames):
        """
        Private method to filter a list of file names by site.
        
        @param filenames list of file names to be filtered (list of string or QString)
        @return file names belonging to the current site (list of string)
        """
        site = self.__site()
        nfilenames = []
        for filename in filenames:
            filename = unicode(filename)
            if filename.startswith(site + os.sep):
                nfilenames.append(filename)
        
        return nfilenames
    
    def extractMessages(self):
        """
        Public method to extract the messages catalog template file.
        """
        title = self.trUtf8("Extract messages")
        
        args = QStringList()
        args.append(self.__getPythonCommand())
        args.append("setup.py")
        args.append("extract_messages")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("\nMessages extracted successfully."))
        res = dia.startProcess(args, wd)
        if res:
            dia.exec_()
            
            try:
                lowerSite = self.__site().lower()
            except PylonsNoSiteSelectedException:
                KQMessageBox.warning(None,
                    title,
                    self.trUtf8('No current site selected or no site created yet.'
                                ' Aborting...'))
                return
            self.__e4project.appendFile(os.path.join(
                wd, lowerSite, "i18n", "%s.pot" % lowerSite))
    
    def __projectLanguageAdded(self, code):
        """
        Private slot handling the addition of a new language.
        
        @param code language code of the new language (string or QString)
        """
        title = self.trUtf8("Initializing message catalog for '%1'").arg(code)
        
        args = QStringList()
        args.append(self.__getPythonCommand())
        args.append("setup.py")
        args.append("init_catalog")
        args.append("-l")
        args.append(code)
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("\nMessage catalog initialized successfully."))
        res = dia.startProcess(args, wd)
        if res:
            dia.exec_()
            
            langFile = self.__e4project.pdata["TRANSLATIONPATTERN"][0]\
                .replace("%language%", code)
            self.__e4project.appendFile(langFile)
    
    def compileCatalogs(self, filenames):
        """
        Public method to compile the message catalogs.
        
        @param filenames list of filenames (not used)
        """
        title = self.trUtf8("Compiling message catalogs")
        
        args = QStringList()
        args.append(self.__getPythonCommand())
        args.append("setup.py")
        args.append("compile_catalog")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("\nMessage catalogs compiled successfully."))
        res = dia.startProcess(args, wd)
        if res:
            dia.exec_()
            
            for entry in os.walk(self.__sitePath()):
                for fileName in entry[2]:
                    fullName = os.path.join(entry[0], fileName)
                    if fullName.endswith('.mo'):
                        self.__e4project.appendFile(fullName)
    
    def compileSelectedCatalogs(self, filenames):
        """
        Public method to update the message catalogs.
        
        @param filenames list of filenames
        """
        title = self.trUtf8("Compiling message catalogs")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        argsLists = []
        
        for filename in self.__normalizeList(self.__siteFilteredList(filenames)):
            locale = self.__getLocale(filename)
            if locale:
                args = QStringList()
                args.append(self.__getPythonCommand())
                args.append("setup.py")
                args.append("compile_catalog")
                args.append("-l")
                args.append(locale)
                argsLists.append(args)
        
        if len(argsLists) == 0:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No locales detected.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("\nMessage catalogs compiled successfully."))
        res = dia.startBatchProcesses(argsLists, wd)
        if res:
            dia.exec_()
            
            for entry in os.walk(self.__sitePath()):
                for fileName in entry[2]:
                    fullName = os.path.join(entry[0], fileName)
                    if fullName.endswith('.mo'):
                        self.__e4project.appendFile(fullName)
    
    def updateCatalogs(self, filenames):
        """
        Public method to update the message catalogs.
        
        @param filenames list of filenames (not used)
        """
        title = self.trUtf8("Updating message catalogs")
        
        args = QStringList()
        args.append(self.__getPythonCommand())
        args.append("setup.py")
        args.append("update_catalog")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("\nMessage catalogs updated successfully."))
        res = dia.startProcess(args, wd)
        if res:
            dia.exec_()
    
    def updateSelectedCatalogs(self, filenames):
        """
        Public method to update the message catalogs.
        
        @param filenames list of filenames
        """
        title = self.trUtf8("Updating message catalogs")
        
        try:
            wd = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        argsLists = []
        
        for filename in self.__normalizeList(self.__siteFilteredList(filenames)):
            locale = self.__getLocale(filename)
            if locale:
                args = QStringList()
                args.append(self.__getPythonCommand())
                args.append("setup.py")
                args.append("update_catalog")
                args.append("-l")
                args.append(locale)
                argsLists.append(args)
        
        if len(argsLists) == 0:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No locales detected.'
                            ' Aborting...'))
            return
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("\nMessage catalogs updated successfully."))
        res = dia.startBatchProcesses(argsLists, wd)
        if res:
            dia.exec_()
    
    ##################################################################
    ## slots below implement sphinx related functions
    ##################################################################
    
    def __sphinxQuickstart(self):
        """
        Private slot to initialize the documentation using "sphinx-quickstart".
        """
        try:
            sitePath = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        spawning, consoleCmd = self.isSpawningConsole(
            unicode(self.__plugin.getPreferences("ConsoleCommand")))
        if consoleCmd:
            docsDir = os.path.join(sitePath, "docs")
            if not os.path.exists(docsDir):
                os.makedirs(docsDir)
            indexFile = os.path.join(docsDir, "index.txt")
            if os.path.exists(indexFile):
                os.remove(indexFile)
            
            proc = QProcess()
            args = Utilities.parseOptionString(consoleCmd)
            args[0] = Utilities.getExecutablePath(args[0])
            args.append(self.__getSphinxCommand(True))
            proc.setWorkingDirectory(docsDir)
            proc.start(args[0], args[1:])
            started = proc.waitForStarted(5000) # wait 5 s
            if not started:
                KQMessageBox.critical(None,
                    self.trUtf8('Process Generation Error'),
                    self.trUtf8('The Sphinx process could not be started.'))
                return
            
            if spawning:
                KQMessageBox.information(None,
                    self.trUtf8("Spawning Console"),
                    self.trUtf8("""Acknowledge this, when the command run in the"""
                                """ console has finished."""))
            else:
                proc.waitForFinished(-1)    # wait until the process finished
            
            # search for files created by sphinx and add them to the project
            for entry in os.walk(docsDir):
                for fileName in entry[2]:
                    fullName = os.path.join(entry[0], fileName)
                    self.__e4project.appendFile(fullName)
    
    def __sphinxBuildHtml(self):
        """
        Private slot to build the HTML documentation using "sphinx-build".
        """
        title = self.trUtf8("Build HTML documentation")
        
        try:
            sitePath = self.__sitePath()
        except PylonsNoSiteSelectedException:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No current site selected or no site created yet.'
                            ' Aborting...'))
            return
        
        docsDir = os.path.join(sitePath, "docs")
        if not os.path.exists(docsDir):
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No documentation directory exists.'
                            ' Use the "initialize" command first.'
                            ' Aborting...'))
            return
        
        buildDir = None
        for entry in os.listdir(docsDir):
            if entry[1:] == "build":
                buildDir = entry
                break
        if buildDir is None:
            KQMessageBox.warning(None,
                title,
                self.trUtf8('No build directory exists.'
                            ' Use the "initialize" command with combined'
                            ' source and build directories.'
                            ' Aborting...'))
            return
        
        args = QStringList()
        args.append(self.__getSphinxCommand())
        args.append("-b")
        args.append("html")
        args.append(".")
        args.append(buildDir)
        
        dia = PylonsDialog(title, 
            msgSuccess = \
                self.trUtf8("\nHTML documentation built successfully."))
        res = dia.startProcess(args, docsDir)
        if res:
            dia.exec_()
            
            # search for files created by sphinx and add them to the project
            for entry in os.walk(docsDir):
                for fileName in entry[2]:
                    fullName = os.path.join(entry[0], fileName)
                    self.__e4project.appendFile(fullName)
