# -*- coding: utf-8 -*-

# Copyright (c) 2009 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a dialog starting a process and showing its output.
"""

import os
import sys

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from KdeQt import KQMessageBox

from Ui_PylonsDialog import Ui_PylonsDialog

import Preferences

class PylonsDialog(QDialog, Ui_PylonsDialog):
    """
    Module implementing a dialog starting a process and showing its output.
    
    It starts a QProcess and displays a dialog that
    shows the output of the process. The dialog is modal,
    which causes a synchronized execution of the process.
    """
    def __init__(self, text, fixed = False, linewrap = True, 
                 msgSuccess = None, msgError = None, 
                 parent = None):
        """
        Constructor
        
        @param text text to be shown by the label (string or QString)
        @keyparam fixed flag indicating a fixed font should be used (boolean)
        @keyparam linewrap flag indicating to wrap long lines (boolean)
        @keyparam msgSuccess optional string to show upon successful execution
            (string or QString)
        @keyparam msgError optional string to show upon unsuccessful execution
            (string or QString)
        @keyparam parent parent widget (QWidget)
        """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        
        self.buttonBox.button(QDialogButtonBox.Close).setEnabled(False)
        self.buttonBox.button(QDialogButtonBox.Cancel).setDefault(True)
        
        self.ioEncoding = str(Preferences.getSystem("IOEncoding"))
        
        self.proc = None
        self.argsLists = []
        self.workingDir = None
        self.msgSuccess = msgSuccess
        self.msgError = msgError
        
        self.outputGroup.setTitle(text)
        
        if fixed:
            if sys.platform == "win32":
                self.resultbox.setFontFamily("Lucida Console")
            else:
                self.resultbox.setFontFamily("Monospace")
        
        if not linewrap:
            self.resultbox.setLineWrapMode(QTextEdit.NoWrap)
    
    def __finish(self):
        """
        Private slot called when the process finished or the user pressed the button.
        """
        if self.proc is not None and \
           self.proc.state() != QProcess.NotRunning:
            self.proc.terminate()
            QTimer.singleShot(2000, self.proc, SLOT('kill()'))
            self.proc.waitForFinished(3000)
        
        self.buttonBox.button(QDialogButtonBox.Close).setEnabled(True)
        self.buttonBox.button(QDialogButtonBox.Cancel).setEnabled(False)
        self.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
        
        self.proc = None
        
        if self.argsLists:
            args = self.argsLists[0][:]
            del self.argsLists[0]
            self.startProcess(args, self.workingDir)
    
    def on_buttonBox_clicked(self, button):
        """
        Private slot called by a button of the button box clicked.
        
        @param button button that was clicked (QAbstractButton)
        """
        if button == self.buttonBox.button(QDialogButtonBox.Close):
            self.close()
        elif button == self.buttonBox.button(QDialogButtonBox.Cancel):
            self.__finish()
    
    def __procFinished(self, exitCode, exitStatus):
        """
        Private slot connected to the finished signal.
        
        @param exitCode exit code of the process (integer)
        @param exitStatus exit status of the process (QProcess.ExitStatus)
        """
        self.normal = (exitStatus == QProcess.NormalExit) and (exitCode == 0)
        self.__finish()
        if self.normal and self.msgSuccess:
            self.resultbox.insertPlainText(self.msgSuccess)
        elif not self.normal and self.msgError:
            self.resultbox.insertPlainText(self.msgError)
        self.resultbox.ensureCursorVisible()
    
    def startProcess(self, args, workingDir = None, showCommand = True):
        """
        Public slot used to start the process.
        
        @param args list of arguments for the process (QStringList)
        @param workingDir working directory for the process (string or QString)
        @param showCommand flag indicating to show the command executed (boolean)
        @return flag indicating a successful start of the process (boolean)
        """
        self.errorGroup.hide()
        
        self.normal = False
        
        self.proc = QProcess()
        
        if showCommand:
            self.resultbox.append(args.join(' '))
            self.resultbox.append('')
        
        self.connect(self.proc, SIGNAL('finished(int, QProcess::ExitStatus)'),
            self.__procFinished)
        self.connect(self.proc, SIGNAL('readyReadStandardOutput()'),
            self.__readStdout)
        self.connect(self.proc, SIGNAL('readyReadStandardError()'),
            self.__readStderr)
        
        if workingDir:
            self.proc.setWorkingDirectory(workingDir)
            self.workingDir = workingDir
        else:
            self.workingDir = QString()
        
        prog = args[0]
        del args[0]
        self.proc.start(prog, args)
        procStarted = self.proc.waitForStarted()
        if not procStarted:
            self.buttonBox.setFocus()
            KQMessageBox.critical(None,
                self.trUtf8('Process Generation Error'),
                self.trUtf8(
                    'The process %1 could not be started. '
                    'Ensure, that it is in the search path.'
                ).arg(prog))
        return procStarted
    
    def startBatchProcesses(self, argsLists, workingDir = None):
        """
        Public slot used to start a batch of processes.
        
        @param argsLists list of lists of arguments for the processes
            (list of QStringList)
        @param workingDir working directory for the process (string or QString)
        @return flag indicating a successful start of the first process (boolean)
        """
        self.argsLists = argsLists[:]
        self.workingDir = workingDir
        
        # start the first process
        args = self.argsLists[0][:]
        del self.argsLists[0]
        res = self.startProcess(args, self.workingDir)
        if not res:
            self.argsLists = []
        
        return res
    
    def normalExit(self):
        """
        Public method to check for a normal process termination.
        
        @return flag indicating normal process termination (boolean)
        """
        return self.normal
    
    def __readStdout(self):
        """
        Private slot to handle the readyReadStdout signal. 
        
        It reads the output of the process, formats it and inserts it into
        the contents pane.
        """
        if self.proc is not None:
            s = unicode(self.proc.readAllStandardOutput(), self.ioEncoding, 'replace')
            self.resultbox.insertPlainText(s)
            self.resultbox.ensureCursorVisible()
    
    def __readStderr(self):
        """
        Private slot to handle the readyReadStderr signal.
        
        It reads the error output of the process and inserts it into the
        error pane.
        """
        if self.proc is not None:
            self.errorGroup.show()
            s = unicode(self.proc.readAllStandardError(), self.ioEncoding, 'replace')
            self.errors.insertPlainText(s)
            self.errors.ensureCursorVisible()
